@@ -1,11 +1,77 @@
+"""
+This file contains a minimal set of tests for compliance with the extension
+array interface test suite, and should contain no other tests.
+The test suite for the full functionality of the array is located in
+`pandas/tests/arrays/`.
+
+The tests in this file are inherited from the BaseExtensionTests, and only
+minimal tweaks should be applied to get the tests passing (by overwriting a
+parent method).
+
+Additional tests should either be added to one of the BaseExtensionTests
+classes (if they are relevant for the extension interface for all dtypes), or
+be added to the array-specific tests in `pandas/tests/arrays/`.
+
+"""
 import numpy as np
 import pytest
 
+import pandas.util._test_decorators as td
+
+from pandas.core.dtypes.cast import can_hold_element
+from pandas.core.dtypes.dtypes import (
+    ExtensionDtype,
+    PandasDtype,
+)
+from pandas.core.dtypes.generic import ABCPandasArray
+
 import pandas as pd
 import pandas._testing as tm
-from pandas.core.arrays.numpy_ import PandasArray, PandasDtype
+from pandas.core.arrays.numpy_ import PandasArray
+from pandas.core.internals import (
+    blocks,
+    managers,
+)
+from pandas.tests.extension import base
 
-from . import base
+# TODO(ArrayManager) PandasArray
+pytestmark = td.skip_array_manager_not_yet_implemented
+
+
+def _extract_array_patched(obj):
+    if isinstance(obj, (pd.Index, pd.Series)):
+        obj = obj._values
+    if isinstance(obj, ABCPandasArray):
+        # TODO for reasons unclear, we get here in a couple of tests
+        #  with PandasArray._typ *not* patched
+        obj = obj.to_numpy()
+
+    return obj
+
+
+def _can_hold_element_patched(obj, element) -> bool:
+    if isinstance(element, PandasArray):
+        element = element.to_numpy()
+    return can_hold_element(obj, element)
+
+
+orig_assert_attr_equal = tm.assert_attr_equal
+
+
+def _assert_attr_equal(attr: str, left, right, obj: str = "Attributes"):
+    """
+    patch tm.assert_attr_equal so PandasDtype("object") is closed enough to
+    np.dtype("object")
+    """
+    if attr == "dtype":
+        lattr = getattr(left, "dtype", None)
+        rattr = getattr(right, "dtype", None)
+        if isinstance(lattr, PandasDtype) and not isinstance(rattr, PandasDtype):
+            left = left.astype(lattr.numpy_dtype)
+        elif isinstance(rattr, PandasDtype) and not isinstance(lattr, PandasDtype):
+            right = right.astype(rattr.numpy_dtype)
+
+    orig_assert_attr_equal(attr, left, right, obj)
 
 
 @pytest.fixture(params=["float", "object"])
@@ -32,6 +98,9 @@ def allow_in_pandas(monkeypatch):
     """
     with monkeypatch.context() as m:
         m.setattr(PandasArray, "_typ", "extension")
+        m.setattr(managers, "_extract_array", _extract_array_patched)
+        m.setattr(blocks, "can_hold_element", _can_hold_element_patched)
+        m.setattr(tm.asserters, "assert_attr_equal", _assert_attr_equal)
         yield
 
 
@@ -106,7 +175,7 @@ def data_for_grouping(allow_in_pandas, dtype):
 
 
 @pytest.fixture
-def skip_numpy_object(dtype):
+def skip_numpy_object(dtype, request):
     """
     Tests for PandasArray with nested data. Users typically won't create
     these objects via `pd.array`, but they can show up through `.array`
@@ -117,14 +186,25 @@ def skip_numpy_object(dtype):
     marker to either an individual test or a test class.
     """
     if dtype == "object":
-        raise pytest.skip("Skipping for object dtype.")
+        mark = pytest.mark.xfail(reason="Fails for object dtype")
+        request.node.add_marker(mark)
 
 
 skip_nested = pytest.mark.usefixtures("skip_numpy_object")
 
 
 class BaseNumPyTests:
-    pass
+    @classmethod
+    def assert_series_equal(cls, left, right, *args, **kwargs):
+        # base class tests hard-code expected values with numpy dtypes,
+        #  whereas we generally want the corresponding PandasDtype
+        if (
+            isinstance(right, pd.Series)
+            and not isinstance(right.dtype, ExtensionDtype)
+            and isinstance(left.dtype, PandasDtype)
+        ):
+            right = right.astype(PandasDtype(right.dtype))
+        return tm.assert_series_equal(left, right, *args, **kwargs)
 
 
 class TestCasting(BaseNumPyTests, base.BaseCastingTests):
@@ -133,12 +213,6 @@ class TestCasting(BaseNumPyTests, base.BaseCastingTests):
         # ValueError: setting an array element with a sequence
         super().test_astype_str(data)
 
-    @skip_nested
-    def test_astype_string(self, data):
-        # GH-33465
-        # ValueError: setting an array element with a sequence
-        super().test_astype_string(data)
-
 
 class TestConstructors(BaseNumPyTests, base.BaseConstructorsTests):
     @pytest.mark.skip(reason="We don't register our dtype")
@@ -146,11 +220,6 @@ class TestConstructors(BaseNumPyTests, base.BaseConstructorsTests):
     def test_from_dtype(self, data):
         pass
 
-    @skip_nested
-    def test_array_from_scalars(self, data):
-        # ValueError: PandasArray must be 1-dimensional.
-        super().test_array_from_scalars(data)
-
     @skip_nested
     def test_series_constructor_scalar_with_index(self, data, dtype):
         # ValueError: Length of passed values is 1, index implies 3.
@@ -170,27 +239,18 @@ class TestGetitem(BaseNumPyTests, base.BaseGetitemTests):
         # AssertionError
         super().test_getitem_scalar(data)
 
-    @skip_nested
-    def test_take_series(self, data):
-        # ValueError: PandasArray must be 1-dimensional.
-        super().test_take_series(data)
-
-    def test_loc_iloc_frame_single_dtype(self, data, request):
-        npdtype = data.dtype.numpy_dtype
-        if npdtype == object:
-            # GH#33125
-            mark = pytest.mark.xfail(
-                reason="GH#33125 astype doesn't recognize data.dtype"
-            )
-            request.node.add_marker(mark)
-        super().test_loc_iloc_frame_single_dtype(data)
-
 
 class TestGroupby(BaseNumPyTests, base.BaseGroupbyTests):
-    @skip_nested
     def test_groupby_extension_apply(
         self, data_for_grouping, groupby_apply_op, request
     ):
+        dummy = groupby_apply_op([None])
+        if (
+            isinstance(dummy, pd.Series)
+            and data_for_grouping.dtype.numpy_dtype == object
+        ):
+            mark = pytest.mark.xfail(reason="raises in MultiIndex construction")
+            request.node.add_marker(mark)
         super().test_groupby_extension_apply(data_for_grouping, groupby_apply_op)
 
 
@@ -202,37 +262,11 @@ class TestInterface(BaseNumPyTests, base.BaseInterfaceTests):
 
 
 class TestMethods(BaseNumPyTests, base.BaseMethodsTests):
-    @pytest.mark.skip(reason="TODO: remove?")
-    def test_value_counts(self, all_data, dropna):
-        pass
-
-    @pytest.mark.xfail(reason="not working. will be covered by #32028")
-    def test_value_counts_with_normalize(self, data):
-        return super().test_value_counts_with_normalize(data)
-
-    @pytest.mark.skip(reason="Incorrect expected")
-    # We have a bool dtype, so the result is an ExtensionArray
-    # but expected is not
-    def test_combine_le(self, data_repeated):
-        super().test_combine_le(data_repeated)
-
-    @skip_nested
-    def test_combine_add(self, data_repeated):
-        # Not numeric
-        super().test_combine_add(data_repeated)
-
     @skip_nested
     def test_shift_fill_value(self, data):
         # np.array shape inference. Shift implementation fails.
         super().test_shift_fill_value(data)
 
-    @skip_nested
-    @pytest.mark.parametrize("box", [pd.Series, lambda x: x])
-    @pytest.mark.parametrize("method", [lambda x: x.unique(), pd.unique])
-    def test_unique(self, data, box, method):
-        # Fails creating expected
-        super().test_unique(data, box, method)
-
     @skip_nested
     def test_fillna_copy_frame(self, data_missing):
         # The "scalar" for this array isn't a scalar.
@@ -243,65 +277,51 @@ class TestMethods(BaseNumPyTests, base.BaseMethodsTests):
         # The "scalar" for this array isn't a scalar.
         super().test_fillna_copy_series(data_missing)
 
-    @skip_nested
-    def test_hash_pandas_object_works(self, data, as_frame):
-        # ndarray of tuples not hashable
-        super().test_hash_pandas_object_works(data, as_frame)
-
     @skip_nested
     def test_searchsorted(self, data_for_sorting, as_series):
         # Test setup fails.
         super().test_searchsorted(data_for_sorting, as_series)
 
-    @skip_nested
-    def test_where_series(self, data, na_value, as_frame):
-        # Test setup fails.
-        super().test_where_series(data, na_value, as_frame)
-
-    @skip_nested
-    @pytest.mark.parametrize("repeats", [0, 1, 2, [1, 2, 3]])
-    def test_repeat(self, data, repeats, as_series, use_numpy):
-        # Fails creating expected
-        super().test_repeat(data, repeats, as_series, use_numpy)
-
     @pytest.mark.xfail(reason="PandasArray.diff may fail on dtype")
     def test_diff(self, data, periods):
         return super().test_diff(data, periods)
 
-    @skip_nested
-    @pytest.mark.parametrize("box", [pd.array, pd.Series, pd.DataFrame])
-    def test_equals(self, data, na_value, as_series, box):
-        # Fails creating with _from_sequence
-        super().test_equals(data, na_value, as_series, box)
 
-
-@skip_nested
 class TestArithmetics(BaseNumPyTests, base.BaseArithmeticOpsTests):
     divmod_exc = None
     series_scalar_exc = None
     frame_scalar_exc = None
     series_array_exc = None
 
+    @skip_nested
+    def test_divmod(self, data):
+        super().test_divmod(data)
+
+    @skip_nested
     def test_divmod_series_array(self, data):
-        s = pd.Series(data)
-        self._check_divmod_op(s, divmod, data, exc=None)
-
-    @pytest.mark.skip("We implement ops")
-    def test_error(self, data, all_arithmetic_operators):
-        pass
+        ser = pd.Series(data)
+        self._check_divmod_op(ser, divmod, data, exc=None)
 
+    @skip_nested
     def test_arith_series_with_scalar(self, data, all_arithmetic_operators):
         super().test_arith_series_with_scalar(data, all_arithmetic_operators)
 
-    def test_arith_series_with_array(self, data, all_arithmetic_operators):
+    def test_arith_series_with_array(self, data, all_arithmetic_operators, request):
+        opname = all_arithmetic_operators
+        if data.dtype.numpy_dtype == object and opname not in ["__add__", "__radd__"]:
+            mark = pytest.mark.xfail(reason="Fails for object dtype")
+            request.node.add_marker(mark)
         super().test_arith_series_with_array(data, all_arithmetic_operators)
 
+    @skip_nested
+    def test_arith_frame_with_scalar(self, data, all_arithmetic_operators):
+        super().test_arith_frame_with_scalar(data, all_arithmetic_operators)
+
 
 class TestPrinting(BaseNumPyTests, base.BasePrintingTests):
     pass
 
 
-@skip_nested
 class TestNumericReduce(BaseNumPyTests, base.BaseNumericReduceTests):
     def check_reduce(self, s, op_name, skipna):
         result = getattr(s, op_name)(skipna=skipna)
@@ -309,6 +329,10 @@ class TestNumericReduce(BaseNumPyTests, base.BaseNumericReduceTests):
         expected = getattr(s.astype(s.dtype._dtype), op_name)(skipna=skipna)
         tm.assert_almost_equal(result, expected)
 
+    @pytest.mark.parametrize("skipna", [True, False])
+    def test_reduce_series(self, data, all_boolean_reductions, skipna):
+        super().test_reduce_series(data, all_boolean_reductions, skipna)
+
 
 @skip_nested
 class TestBooleanReduce(BaseNumPyTests, base.BaseBooleanReduceTests):
@@ -316,16 +340,6 @@ class TestBooleanReduce(BaseNumPyTests, base.BaseBooleanReduceTests):
 
 
 class TestMissing(BaseNumPyTests, base.BaseMissingTests):
-    @skip_nested
-    def test_fillna_scalar(self, data_missing):
-        # Non-scalar "scalar" values.
-        super().test_fillna_scalar(data_missing)
-
-    @skip_nested
-    def test_fillna_series_method(self, data_missing, fillna_method):
-        # Non-scalar "scalar" values.
-        super().test_fillna_series_method(data_missing, fillna_method)
-
     @skip_nested
     def test_fillna_series(self, data_missing):
         # Non-scalar "scalar" values.
@@ -336,68 +350,15 @@ class TestMissing(BaseNumPyTests, base.BaseMissingTests):
         # Non-scalar "scalar" values.
         super().test_fillna_frame(data_missing)
 
-    @pytest.mark.skip("Invalid test")
-    def test_fillna_fill_other(self, data):
-        # inplace update doesn't work correctly with patched extension arrays
-        # extract_array returns PandasArray, while dtype is a numpy dtype
-        super().test_fillna_fill_other(data_missing)
-
 
 class TestReshaping(BaseNumPyTests, base.BaseReshapingTests):
-    @pytest.mark.skip("Incorrect parent test")
-    # not actually a mixed concat, since we concat int and int.
-    def test_concat_mixed_dtypes(self, data):
-        super().test_concat_mixed_dtypes(data)
-
-    @pytest.mark.xfail(
-        reason="GH#33125 PandasArray.astype does not recognize PandasDtype"
-    )
-    def test_concat(self, data, in_frame):
-        super().test_concat(data, in_frame)
-
-    @pytest.mark.xfail(
-        reason="GH#33125 PandasArray.astype does not recognize PandasDtype"
-    )
-    def test_concat_all_na_block(self, data_missing, in_frame):
-        super().test_concat_all_na_block(data_missing, in_frame)
-
-    @skip_nested
+    @pytest.mark.skip(reason="Incorrect expected.")
     def test_merge(self, data, na_value):
-        # Fails creating expected
+        # Fails creating expected (key column becomes a PandasDtype because)
         super().test_merge(data, na_value)
 
-    @skip_nested
-    def test_merge_on_extension_array(self, data):
-        # Fails creating expected
-        super().test_merge_on_extension_array(data)
-
-    @skip_nested
-    def test_merge_on_extension_array_duplicates(self, data):
-        # Fails creating expected
-        super().test_merge_on_extension_array_duplicates(data)
-
-    @skip_nested
-    def test_transpose_frame(self, data):
-        super().test_transpose_frame(data)
-
 
 class TestSetitem(BaseNumPyTests, base.BaseSetitemTests):
-    @skip_nested
-    def test_setitem_scalar_series(self, data, box_in_series):
-        # AssertionError
-        super().test_setitem_scalar_series(data, box_in_series)
-
-    @skip_nested
-    def test_setitem_sequence(self, data, box_in_series):
-        # ValueError: shape mismatch: value array of shape (2,1) could not
-        # be broadcast to indexing result of shape (2,)
-        super().test_setitem_sequence(data, box_in_series)
-
-    @skip_nested
-    def test_setitem_sequence_mismatched_length_raises(self, data, as_array):
-        # ValueError: PandasArray must be 1-dimensional.
-        super().test_setitem_sequence_mismatched_length_raises(data, as_array)
-
     @skip_nested
     def test_setitem_sequence_broadcasts(self, data, box_in_series):
         # ValueError: cannot set using a list-like indexer with a different
@@ -451,7 +412,6 @@ class TestSetitem(BaseNumPyTests, base.BaseSetitemTests):
     def test_setitem_mask(self, data, mask, box_in_series):
         super().test_setitem_mask(data, mask, box_in_series)
 
-    @skip_nested
     def test_setitem_mask_raises(self, data, box_in_series):
         super().test_setitem_mask_raises(data, box_in_series)
 
@@ -464,7 +424,6 @@ class TestSetitem(BaseNumPyTests, base.BaseSetitemTests):
     def test_setitem_integer_array(self, data, idx, box_in_series):
         super().test_setitem_integer_array(data, idx, box_in_series)
 
-    @skip_nested
     @pytest.mark.parametrize(
         "idx, box_in_series",
         [
@@ -486,7 +445,28 @@ class TestSetitem(BaseNumPyTests, base.BaseSetitemTests):
     def test_setitem_loc_iloc_slice(self, data):
         super().test_setitem_loc_iloc_slice(data)
 
+    def test_setitem_with_expansion_dataframe_column(self, data, full_indexer):
+        # https://github.com/pandas-dev/pandas/issues/32395
+        df = expected = pd.DataFrame({"data": pd.Series(data)})
+        result = pd.DataFrame(index=df.index)
+
+        # because result has object dtype, the attempt to do setting inplace
+        #  is successful, and object dtype is retained
+        key = full_indexer(df)
+        result.loc[key, "data"] = df["data"]
+
+        # base class method has expected = df; PandasArray behaves oddly because
+        #  we patch _typ for these tests.
+        if data.dtype.numpy_dtype != object:
+            if not isinstance(key, slice) or key != slice(None):
+                expected = pd.DataFrame({"data": data.to_numpy()})
+        self.assert_frame_equal(result, expected)
+
 
 @skip_nested
 class TestParsing(BaseNumPyTests, base.BaseParsingTests):
     pass
+
+
+class Test2DCompat(BaseNumPyTests, base.Dim2CompatTests):
+    pass
